// 
// このコードは、https://demo.mescius.jp/diodocs/pdf のサンプルの一部として提供されています。
// © MESCIUS inc. All rights reserved.
// 
using System;
using System.IO;
using System.Drawing;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Text;
using GrapeCity.Documents.Drawing;

namespace DsPdfWeb.Demos.Basics
{
    // このサンプルは、ソフトハイフン（0x00AD）を含むテキストをハイフネーションする方法を示しています。
    // TextLayout.WrapModeがWordWrapに設定されている場合、ブレークはソフトハイフンの位置（テキスト内に
    // 存在する場合）に挿入されます。ハイフネーションを制御するために2つのプロパティが用意されています。
    // - TextLayout.SoftHyphenReplacementCharCode: 行をまたいで単語を改行する際にソフトハイフンの
    //   代わりに使われるキャラクタを指定します。このプロパティのデフォルトは、0x002D（Unicodeの
    //   ハイフンマイナス文字）が設定されています。このプロパティを0に設定すると、 ハイフンキャラクタを
    //   一切表示せずに単語を区切ることができます。このプロパティを-1に設定すると、ソフトハイフンでの
    //   改行を防ぐことができます。
    // - TextLayout.LinesBetweenConsecutiveHyphens: ハイフンで終わる行間の非ハイフン付き行の最小数を
    //   指定します。このプロパティのデフォルトはゼロです。
    public class Hyphenation
    {
        public int CreatePDF(Stream stream)
        {
            // こちら http://www.plazoo.com/en/tools.asp のツールを使用し WordCharWrap のサンプルテキストにソフトハイフンを挿入しました。
            var str =
                "Lose noth&shy;ing in your doc&shy;u&shy;ments! Grape&shy;City Doc&shy;u&shy;ments for PDF " +
                "in&shy;cludes text and para&shy;graph format&shy;ting, spe&shy;cial char&shy;ac&shy;ters, " +
                "mul&shy;tiple lan&shy;guages, RTL sup&shy;port, ver&shy;tic&shy;al and ro&shy;tated text " +
                "on all sup&shy;por&shy;ted plat&shy;forms.";
            // HTMLのソフトハイフンをUnicodeのものに置き換えます。
            str = str.Replace("&shy;", "\u00AD");

            var doc = new GcPdfDocument();
            var page = doc.NewPage();
            var g = page.Graphics;

            var tl = g.CreateTextLayout();
            tl.Append(str);
            tl.DefaultFormat.Font = Common.Util.getFont();
            tl.DefaultFormat.FontSize = 13;
            tl.MaxWidth = 72 * 3;

            // デフォルトでは、ソフトハイフン（0x00AD）で改行する場合は、0x002D（ハイフンマイナス）が
            // ハイフネーション文字として使用されます。
            tl.PerformLayout(true);

            var dy = tl.Lines[0].Height + 72 / 16;
            var rc = new RectangleF(72, 72 + dy, tl.MaxWidth.Value, 72 * 1.4F);

            g.DrawString("デフォルトのハイフネーション：", tl.DefaultFormat, new PointF(rc.Left, rc.Top - dy));
            g.DrawTextLayout(tl, rc.Location);
            g.DrawRectangle(rc, Color.CornflowerBlue);

            rc.Offset(0, 72 * 2);
            // これにより、2つの連続した行がハイフンで結ばれることを避けることができます。
            tl.LinesBetweenConsecutiveHyphens = 1;
            // ハイフネーションのオプションを変更するには RecalculateGlyphs() が必要です。
            tl.PerformLayout(true);
            g.DrawString("LinesBetweenConsecutiveHyphens: 1", tl.DefaultFormat, new PointF(rc.Left, rc.Top - dy));
            g.DrawTextLayout(tl, rc.Location);
            g.DrawRectangle(rc, Color.CornflowerBlue);

            rc.Offset(0, 72 * 2);
            // 前の設定をリセットします。
            tl.LinesBetweenConsecutiveHyphens = 0;
            // 単語のハイフネーションを完全に防ぎます。
            tl.SoftHyphenReplacementCharCode = -1;
            // ハイフネーションのオプションを変更するには RecalculateGlyphs() が必要です。
            tl.PerformLayout(true);
            g.DrawString("SoftHyphenReplacementCharCode: -1", tl.DefaultFormat, new PointF(rc.Left, rc.Top - dy));
            g.DrawTextLayout(tl, rc.Location);
            g.DrawRectangle(rc, Color.CornflowerBlue);

            // PDF ドキュメントを保存します。
            doc.Save(stream);
            return doc.Pages.Count;
        }
    }
}
